package buildme

import (
	"bufio"
	"buildme/utils"
	"context"
	"encoding/json"
	"fmt"
	"io"
	"os"
	"strings"

	yaml "gopkg.in/yaml.v2"

	"github.com/docker/docker/api/types"
	docker "github.com/docker/docker/client"
)

// Project type
type Project struct {
	tarFilename string
	isTemp      bool
	//utils.Filer
	//io.ReadCloser // Tar format? tar reader perhaps
	//Name       string
	//Dockerfile string

	conf ProjectConf
	// Meta YML data
}

//NewProject Creates a project
func NewProject() *Project {
	return &Project{
		tarFilename: "",
		isTemp:      false,
		//	Name:       name,
		//Dockerfile: "docker/Dockerfile.build",
		conf: ProjectConf{},
	}
}

func (p *Project) ParseConf(data []byte) error {
	return yaml.Unmarshal(data, &p.conf)
}

//SetTarFile sets the source project tar
func (p *Project) SetTarFile(tarFile string, isTemp bool) *Project {
	p.tarFilename = tarFile
	p.isTemp = isTemp
	return p
}

/*func (p *Project) SetName(name string) *Project {
	p.Name = name
	return p
}
func (p *Project) SetDockerfile(dockerfile string) *Project {
	p.Dockerfile = dockerfile
	return p
}*/

func (p *Project) SetConf(projConf ProjectConf) *Project {
	p.conf = projConf
	return p
}

func (p *Project) Reader() io.ReadCloser {
	f, _ := os.Open(p.tarFilename)
	return f
}

func (p *Project) Close() {
	if p.isTemp {
		log.Println("Closing temporary file")
		os.Remove(p.tarFilename)
	}
	//os.Remove(tarFile) // Should remove??
}

func (p *Project) Task(name string) error {

	log.Printf("Proj: %s Running task: %s", p.conf.Project, name)

	taskconf, ok := p.conf.Tasks[name]
	if !ok {
		return fmt.Errorf("Task '%s' not found\nTasks: %s", name, p.conf.Tasks)
	}
	log.Println("Task found", taskconf)

	// Running task:
	log.Println("Using image:", taskconf.Use)
	imgconf, ok := p.conf.Images[taskconf.Use]
	if !ok {
		return fmt.Errorf("Image '%s' not found\nImages: %s", taskconf.Use, p.conf.Images)
	}

	// Task kind docker
	// Stupid docker cli
	dockerhost := os.Getenv("DOCKER_HOST")
	if dockerhost != "" && !strings.Contains(dockerhost, "://") {
		os.Setenv("DOCKER_HOST", "http://"+dockerhost)
	}

	cli, err := docker.NewEnvClient()
	if err != nil {
		return err
	}
	ctx := context.Background()
	dockerTag := p.conf.Project + "-" + taskconf.Use
	{
		// Check if container exists

		/*r, err := cli.ContainerInspect(ctx, dockertag)
		if err != nil {
			log.Println("Error loading container", err)
		} else {
			log.Println("Container FOUND state:", r.State)
		}*/
		// Execute inside container
		log.Println("===== Building IMAGE")
		res, err := cli.ImageBuild(ctx, p.Reader(), types.ImageBuildOptions{Tags: []string{dockerTag}, Dockerfile: imgconf.Dockerfile, Remove: true})
		if err != nil {
			return err
		}
		defer res.Body.Close()

		// Just a reader and output
		br := bufio.NewReader(res.Body)
		for {
			jdata, _, err := br.ReadLine()
			if err != nil {
				break
			}
			mdata := map[string]interface{}{}
			json.Unmarshal(jdata, &mdata)

			msg, ok := mdata["stream"]
			if !ok {
				msg = mdata["error"] // Error if exists
			}
			log.Println(msg)
		}
		// Just output
	}

	/*containerCfg := &container.Config{
		Image: dockerTag,
		Cmd:   []string{"ls"},
	}
	log.Println("===> Creating container")
	containerRes, err := cli.ContainerCreate(ctx, containerCfg, nil, nil, dockerTag)
	if err != nil {
		return err
	}
	_ = containerRes*/

	log.Println("Executing commands")
	img, err := ImageCli(dockerTag)
	if err != nil {
		return err
	}
	// Builds image
	for _, cmd := range taskconf.Command {
		vcmd := utils.ParseField(cmd)
		var err error
		switch vcmd[0] {
		case "RUN":
			err = img.Run(vcmd[1:])
		case "COPY":
			err = img.Copy(vcmd[1], vcmd[2])
		}
		if err != nil {
			return err
		}
	}

	return nil
}
